@@ -21,13 +21,13 @@
 ##END HEADER
 
 mKrigFastPredict <- function(object, gridList, ynew = NULL,
-                          derivative = 0, Z = NULL, drop.Z = FALSE,
-                          NNSize=5, setupObject= NULL,
-                          giveWarnings=TRUE) 
+                          derivative = 0, ZGrid = NULL, drop.Z = FALSE,
+                          NNSize=4, setupObject= NULL,
+                          giveWarnings=TRUE,
+                          verbose=FALSE) 
                            {
   #NOTE: covariance model is specified by the arguments in object$args
-  #cov.args <- c( object$args, list(...) )
-  
+  # cov.args <- c( object$args, list(...) )
   # For convenience the Z covariates are already assumed to be 
   # in the unrolled form. But this may be awkward if this 
   # function is called directly 
@@ -42,13 +42,12 @@ mKrigFastPredict <- function(object, gridList, ynew = NULL,
  }
                                
   names( gridList)<- c("x","y")
-  
-  np<- NNSize                            
   xObs<- object$x
   
   nx<- length(gridList$x )
   ny<- length(gridList$y )
-  
+  # if new observations then find the new fitted surface
+  # note how the fit object faciliates this.
   if (!is.null(ynew)) {
     coef.hold <- mKrig.coef(object, ynew,
                             collapseFixedEffect=TRUE)
@@ -63,12 +62,19 @@ mKrigFastPredict <- function(object, gridList, ynew = NULL,
   # Tmatrix <- fields.mkpoly(xnew, m=object$m)
   # only do this if nt>0, i.e. there is a fixed part.
   #
+  
   if (!drop.Z & (object$nZ > 0) & (derivative >0) ) {
     stop("derivative not supported with Z covariate included
          use drop.Z = FALSE to omit Z ")
   }
   if( object$nt>0){
+    # create locations from gridList
     xnew<- make.surface.grid( gridList)
+    # unroll ZGrid to a  matrix 
+    #print( summary(ZGrid))
+    #print( summary(gridList))
+    Z<- unrollZGrid( gridList, ZGrid) 
+    
     if (derivative == 0) {
       if (drop.Z | object$nZ == 0) {
         # just evaluate polynomial and not the Z covariate
@@ -88,20 +94,16 @@ mKrigFastPredict <- function(object, gridList, ynew = NULL,
       }
     }
     else {
-      temp1 <- fields.derivative.poly(xnew, m = object$m,
-                                      beta[object$ind.drift,])
+     stop("derivative not implemented with rapid predict algorithm")
     }
+###### end of evaliuating the fixed linear part of the prediction 
+###### note that temp1 here is a vector not an image/matrix. 
    
   }  
   # add nonparametric part. Covariance basis functions
   # times coefficients.
-  
-  
-  # syntax is the name of the function and then a list with
-  # all the arguments. This allows for different covariance functions
-  # that have been passed as their name.
-  
-  # enlarge the grid if needed so that obs have np grid point points on all margins. 
+
+  # enlarge the  evaulation grid if needed so that obs have NNSize grid point points on all margins. 
   
   if( (min(xObs[,1]) < gridList$x[1]) | (max(xObs[,1]) > gridList$x[nx] ) ) {
     stop( "x obs locations can not be outside the grid ")
@@ -109,27 +111,27 @@ mKrigFastPredict <- function(object, gridList, ynew = NULL,
   if( (min(xObs[,2]) < gridList$y[1]) | (max(xObs[,2]) > gridList$y[ny])  ){
     stop( "y obs locations can not be outside the grid ")
   }
-  
+ 
   # adjust grid if needed to include a margin of NNSize+1 grid points beyond xObs
   # these are the slightly larger grids by adding margins.
   # also create sparse matrices. 
-     
   if( is.null(setupObject) ){
+    
   setupObject<- mKrigFastPredictSetup(object, 
                                       gridList = gridList, 
                                         NNSize = NNSize,
-                                  giveWarnings = giveWarnings)
-                                                 
+                                  giveWarnings = giveWarnings,
+                                       verbose = verbose)
   }
-    
-  gridListNew<-  setupObject$marginInfo$gridListNew
+  # the expanded grid to include extra neighbors 
+  gridListNew<-  setupObject$marginInfo$predictionGrid
   nxNew<- length(gridListNew$x )
   nyNew<- length(gridListNew$y )
   # indX and indY are the subset of indices that match gridList
   # ( gridListNew contains the grids in gridList)
-  indX<- setupObject$marginInfo$indX
-  indY<- setupObject$marginInfo$indY
-  
+  indX<- range(setupObject$marginInfo$indexSubset$x)
+  indY<- range(setupObject$marginInfo$indexSubset$y)
+  # 
   if( ((indX[2]- indX[1] + 1)!= nx)| ((indY[2]- indY[1] + 1)!= ny)) {
     cat(" indX, nx") 
     print( c(indX, nx) )
@@ -137,21 +139,34 @@ mKrigFastPredict <- function(object, gridList, ynew = NULL,
     print( c(indY, ny) )
     stop("mismatch between  subset of larger grid and gridList passed")
   }
- 
-    c.coefWghts<-  colSums( diag.spam( c(object$c.coef) ) %*% 
-                              setupObject$offGridObject$B )
-    c.coefWghts<- matrix( c.coefWghts, nxNew, nyNew )
-    
-    
+  # funky  coefficients on the grid that stand in for the actual ones.
+  ################################################################################
+  ################################################################################
+  ################################################################################
+    # the bug found 12:07 AM june 26,2025
+    # c.coefStar<-  colSums( diag.spam( c(object$c.coef) ) %*% 
+    #                           setupObject$offGridObject$B )
+   c.coefStar<-  colSums( diag.spam( c(c.coef) ) %*% 
+                             setupObject$offGridObject$B )
+  
+  ################################################################################
+  ################################################################################
+  ################################################################################
+  
+    # reshape as an image. 
+    c.coefStar<- matrix( c.coefStar, nxNew, nyNew )
+    #
     # fast multiplication of covariances on the grid with
-    # the coefficients on the grid
+    # the coefficients, cStar, on the grid (via FFT)
+    # note that there can be lots of zeroes in coefWghts since the 
+    # approximation is local. 
+     
+    temp2<-  stationaryImageCov( Y=c.coefStar, covObject = setupObject$cov.obj)
     
-    temp2<-  stationary.image.cov( Y=c.coefWghts, cov.obj=setupObject$cov.obj)
-
-    # cut down the size of temp2 trimming off margins using to approximate
+    # cut down the size of temp2 trimming off margins used to approximate
     # exact covariance kernel. 
     temp2<- temp2[ indX[1]:indX[2], indY[1]:indY[2] ]
-    
+  
   # add fixed part and spatial  parts together and coerce to matrix
   if( object$nt>0){
     return( (matrix(temp1,nx,ny) + temp2) )
@@ -161,3 +176,4 @@ mKrigFastPredict <- function(object, gridList, ynew = NULL,
     return(  temp2)
   }
 }
+
