@@ -1,9 +1,9 @@
+#
 # fields  is a package for analysis of spatial data written for
-# the R software environment .
-# Copyright (C) 2016
-# University Corporation for Atmospheric Research (UCAR)
-# Contact: Douglas Nychka, nychka@ucar.edu,
-# National Center for Atmospheric Research, PO Box 3000, Boulder, CO 80307-3000
+# the R software environment.
+# Copyright (C) 2024 Colorado School of Mines
+# 1500 Illinois St., Golden, CO 80401
+# Contact: Douglas Nychka,  douglasnychka@gmail.com,
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
 # You should have received a copy of the GNU General Public License
 # along with the R software environment if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-# or see http://www.r-project.org/Licenses/GPL-2    
+# or see http://www.r-project.org/Licenses/GPL-2
+##END HEADER
 "predict.surface" <- function(object, ...) {
     UseMethod("predict.surface")
 }
@@ -61,31 +62,4 @@ predict.surface.default<- function(object,...){
     return(out)
 }
 
-"predictSurface.mKrig" <- function( object, ...){
-	NextMethod("predictSurface.Krig")
-}
 
-"predictSurface.fastTps" <- function(object, grid.list = NULL, 
-       extrap = FALSE, chull.mask = NA, nx = 80, ny = 80,
-       xy = c(1,2),  verbose = FALSE, ...) {
-# NOTE:  See  predictSurface.default for comments
-    if (is.null(grid.list)) {
-        grid.list <- fields.x.to.grid(object$x, nx = nx, ny = ny, 
-            xy = xy)
-    } 
-# in the case of fastTps pass the grid list instead of the locations of grid points
-#  (see xg in predictSurface.default)
-    out <-  predict(object, grid.list=grid.list, xy=xy, ...)
-    out <-  as.surface(grid.list, out )
-    #
-    # if extrapolate is FALSE set all values outside convex hull to NA
-    if (!extrap) {
-        if (is.na(chull.mask)) {
-            chull.mask <- unique.matrix(object$x[, xy])
-        }
-        xg<- make.surface.grid( grid.list)
-        out$z[!in.poly(xg[, xy], xp = chull.mask, convex.hull = TRUE)] <- NA
-    }
-    #
-    return(out)
-}
