@@ -1,9 +1,9 @@
+#
 # fields  is a package for analysis of spatial data written for
-# the R software environment .
-# Copyright (C) 2018
-# University Corporation for Atmospheric Research (UCAR)
-# Contact: Douglas Nychka, nychka@ucar.edu,
-# National Center for Atmospheric Research, PO Box 3000, Boulder, CO 80307-3000
+# the R software environment.
+# Copyright (C) 2024 Colorado School of Mines
+# 1500 Illinois St., Golden, CO 80401
+# Contact: Douglas Nychka,  douglasnychka@gmail.com,
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,16 +17,17 @@
 # You should have received a copy of the GNU General Public License
 # along with the R software environment if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-# or see http://www.r-project.org/Licenses/GPL-2    
+# or see http://www.r-project.org/Licenses/GPL-2
+##END HEADER
 unrollZGrid<- function( grid.list, ZGrid){
   if( is.null(ZGrid)){
     return(ZGrid)
   }
   if( is.list( ZGrid) ){
-     if( any(grid.list[[1]] != ZGrid[[1]]) |any(grid.list[[2]] != ZGrid[[2]]) ){
+     if( any(grid.list[[1]] != ZGrid[[1]]) | any(grid.list[[2]] != ZGrid[[2]]) ){
          stop("grid list does not match grid for covariates")
        }  
-# wipe out the x and y components of ZGrid because grid.list will be used
+# wipe out the x and y components of ZGrid 
   ZGrid<- ZGrid$z
   }
 # check dimensions
@@ -34,6 +35,7 @@ unrollZGrid<- function( grid.list, ZGrid){
       nx<- length( grid.list[[1]])
       ny<- length( grid.list[[2]])
       if( (Zdim[1] != nx) | (Zdim[2] != ny) ){
+         cat( "Z:", Zdim, "grid", nx, ny, fill=TRUE )
          stop( "Dimension of ZGrid does not match dimensions of location grid list.")
       }
 # reshape as a matrix where rows index locations.
